"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = require("fs");
const path_1 = require("path");
const utils_1 = require("./utils");
const staticServerDefaultOptions = {
    index: 'index.html',
    defer: false,
    maxAge: 0,
    immutable: false,
    hidden: false,
    format: false,
    extensions: false,
    brotli: true,
    gzip: true,
};
/**
 * Send file at `path` with the
 * given `options` to the koa `ctx`.
 *
 * @param {Context} ctx
 * @param {String} path
 * @param {Object} [opts]
 * @return {Function}
 * @api public
 */
function send(ctx, rootPath, opts = {}) {
    return __awaiter(this, void 0, void 0, function* () {
        const root = path_1.normalize(path_1.resolve(rootPath));
        const { index, maxAge, immutable, hidden, format, extensions, brotli, gzip, setHeaders, } = Object.assign({}, staticServerDefaultOptions, opts);
        let path = ctx.path.substr(path_1.parse(ctx.path).root.length);
        // normalize path
        try {
            path = decodeURIComponent(path);
        }
        catch (err) {
            ctx.throw(400, 'faild to decoded', err);
        }
        // index file support
        const trailingSlash = ctx.path[ctx.path.length - 1] === '/';
        if (index && trailingSlash) {
            path += index;
        }
        path = utils_1.resolvePath(ctx, root, path);
        // hidden file support, ignore
        if (!hidden && isHidden(root, path)) {
            return false;
        }
        // serve brotli file when possible otherwise gzipped file when possible
        if (ctx.acceptsEncodings('br', 'deflate', 'identity') === 'br' &&
            brotli &&
            (yield utils_1.exists(path + '.br'))) {
            path = path + '.br';
            ctx.set('Content-Encoding', 'br');
            ctx.res.removeHeader('Content-Length');
        }
        else if (ctx.acceptsEncodings('gzip', 'deflate', 'identity') === 'gzip' &&
            gzip &&
            (yield utils_1.exists(path + '.gz'))) {
            path = path + '.gz';
            ctx.set('Content-Encoding', 'gzip');
            ctx.res.removeHeader('Content-Length');
        }
        if (extensions && !/\..*$/.exec(path)) {
            for (let ext of extensions) {
                if (!/^\./.exec(ext)) {
                    ext = '.' + ext;
                }
                if (yield utils_1.exists(path + ext)) {
                    path = path + ext;
                    break;
                }
            }
        }
        // stat
        let stats;
        try {
            stats = yield utils_1.stat(path);
            // Format the path to serve static file servers
            // and not require a trailing slash for directories,
            // so that you can do both `/directory` and `/directory/`
            if (stats.isDirectory()) {
                if (format && index) {
                    path += '/' + index;
                    stats = yield utils_1.stat(path);
                }
                else {
                    return false;
                }
            }
        }
        catch (err) {
            const notfound = new Set(['ENOENT', 'ENAMETOOLONG', 'ENOTDIR']);
            if (notfound.has(err.code)) {
                ctx.throw(404, err);
            }
            ctx.throw(500, err);
            return false; // TODO remove when typescript fix the control flow detect of never return type
        }
        if (setHeaders) {
            setHeaders(ctx.res, path, stats);
        }
        // stream
        ctx.set('Content-Length', stats.size.toString());
        if (!ctx.response.get('Last-Modified')) {
            ctx.set('Last-Modified', stats.mtime.toUTCString());
        }
        if (!ctx.response.get('Cache-Control')) {
            const directives = ['max-age=' + Math.floor(maxAge / 1000)];
            if (immutable) {
                directives.push('immutable');
            }
            ctx.set('Cache-Control', directives.join(','));
        }
        ctx.type = type(path);
        ctx.body = fs_1.createReadStream(path);
        return path;
    });
}
exports.send = send;
/**
 * Check if it's hidden.
 *
 * @param {string} root
 * @param {string} path
 * @returns
 */
function isHidden(root, path) {
    const filePath = path.substr(root.length).split(path_1.sep);
    for (let i = 0, len = filePath.length; i < len; ++i) {
        if (filePath[i][0] === '.') {
            return true;
        }
    }
    return false;
}
/**
 * File type.
 *
 * @param {string} file
 * @returns
 */
function type(file) {
    return path_1.extname(path_1.basename(file, '.gz'));
}
//# sourceMappingURL=send.js.map